@@ -17,7 +17,7 @@ You should have received a copy of the GNU General Public License
 along with this program.  If not, see <http://www.gnu.org/licenses/>
 */
 
-#include <QtGui> // FIXME: High
+#include <QAbstractItemView>
 
 
 class DumpView: public QAbstractItemView
@@ -39,7 +39,8 @@ protected:
     QRegion visualRegionForSelection( const QItemSelection &selection ) const;
 protected slots:
     void dataChanged( const QModelIndex &topLeft, 
-            const QModelIndex &bottomRight );
+            const QModelIndex &bottomRight,
+            const QVector<int> &roles );
     void selectionChanged( const QItemSelection &selected,
     const QItemSelection &deselected );
     void paintEvent(QPaintEvent *event);
