@@ -97,14 +97,19 @@ PortsWindow::PortsWindow(PortGroupList *pgl, QWidget *parent)
     connect(plm->getPortModel(), SIGNAL(modelReset()), 
         SLOT(when_portModel_reset()));
 
-    connect( tvPortList->selectionModel(), 
+    connect(actionPort_Configuration, SIGNAL(triggered()),
+        SLOT(when_actionPort_Configuration_triggered()));
+    connect(tvPortList, SIGNAL(activated(const QModelIndex&)),
+        SLOT(when_actionPort_Configuration_triggered(const QModelIndex&)));
+    connect(tvPortList->selectionModel(),
         SIGNAL(currentChanged(const QModelIndex&, const QModelIndex&)), 
         this, SLOT(when_portView_currentChanged(const QModelIndex&, 
             const QModelIndex&)));
 
     connect(this,
             SIGNAL(currentPortChanged(const QModelIndex&, const QModelIndex&)),
-            portWidget, SLOT(setCurrentPortIndex(const QModelIndex&)));
+            portWidget,
+            SLOT(setCurrentPortIndex(const QModelIndex&, const QModelIndex&)));
     connect(this,
             SIGNAL(currentPortChanged(const QModelIndex&, const QModelIndex&)),
             streamsWidget, SLOT(setCurrentPortIndex(const QModelIndex&)));
@@ -597,9 +602,11 @@ void PortsWindow::on_actionExclusive_Control_triggered(bool checked)
     }
 }
 
-void PortsWindow::on_actionPort_Configuration_triggered()
+void PortsWindow::when_actionPort_Configuration_triggered(
+        const QModelIndex &portIndex)
 {
-    QModelIndex    current = tvPortList->selectionModel()->currentIndex();
+    QModelIndex current = portIndex.isValid() ?
+        portIndex : tvPortList->selectionModel()->currentIndex();
 
     if (proxyPortModel)
         current = proxyPortModel->mapToSource(current);
@@ -614,6 +621,8 @@ void PortsWindow::on_actionPort_Configuration_triggered()
     // TODO: extend Port::protoDataCopyInto() to accept an optional param
     // which says copy only modifiable fields
     //plm->port(current).protoDataCopyInto(&config);
+    config.set_description(port.systemDescription().toStdString());
+    config.set_user_description(port.userDescription().toStdString());
     config.set_transmit_mode(port.transmitMode());
     config.set_is_tracking_stream_stats(port.trackStreamStats());
     config.set_is_exclusive_control(port.hasExclusiveControl());
