@@ -51,6 +51,8 @@ enum {
     kAvgRxFrameRate,
     kAvgTxBitRate,
     kAvgRxBitRate,
+    kAvgLatency,
+    kAvgJitter,
     kMaxAggrStreamStats
 };
 static QStringList aggrStatTitles = QStringList()
@@ -61,7 +63,9 @@ static QStringList aggrStatTitles = QStringList()
     << "Avg\nTx PktRate"
     << "Avg\nRx PktRate"
     << "Avg\nTx BitRate"
-    << "Avg\nRx BitRate";
+    << "Avg\nRx BitRate"
+    << "Avg\nLatency"
+    << "Avg\nJitter";
 
 static const uint kAggrGuid = 0xffffffff;
 
@@ -164,12 +168,12 @@ QVariant StreamStatsModel::data(const QModelIndex &index, int role) const
             return QString("%L1").arg(aggrGuidStats_.value(guid).txDuration);
         case kAvgTxFrameRate:
             return aggrGuidStats_.value(guid).txDuration <= 0 ? QString("-") :
-                QString("%L1").arg(
+                 XLocale().toPktRateString(
                     aggrGuidStats_.value(guid).txPkts
                         / aggrGuidStats_.value(guid).txDuration);
         case kAvgRxFrameRate:
             return aggrGuidStats_.value(guid).txDuration <= 0 ? QString("-") :
-                 QString("%L1").arg(
+                 XLocale().toPktRateString(
                     aggrGuidStats_.value(guid).rxPkts
                         / aggrGuidStats_.value(guid).txDuration);
         case kAvgTxBitRate:
@@ -184,6 +188,18 @@ QVariant StreamStatsModel::data(const QModelIndex &index, int role) const
                     (aggrGuidStats_.value(guid).rxBytes
                             + 24 * aggrGuidStats_.value(guid).rxPkts) * 8
                         / aggrGuidStats_.value(guid).txDuration);
+        case kAvgLatency:
+            return aggrGuidStats_.value(guid).latencyCount <= 0
+                        || aggrGuidStats_.value(guid).latencySum <= 0 ? QString("-") :
+                XLocale().toTimeIntervalString(
+                    aggrGuidStats_.value(guid).latencySum
+                        / aggrGuidStats_.value(guid).latencyCount);
+        case kAvgJitter:
+            return aggrGuidStats_.value(guid).latencyCount <= 0
+                        || aggrGuidStats_.value(guid).latencySum <= 0 ? QString("-") :
+                XLocale().toTimeIntervalString(
+                    aggrGuidStats_.value(guid).jitterSum
+                        / aggrGuidStats_.value(guid).latencyCount);
         default:
             break;
         };
@@ -258,6 +274,8 @@ void StreamStatsModel::appendStreamStatsList(
         ss.txPkts = s.tx_pkts();
         ss.rxBytes = s.rx_bytes();
         ss.txBytes = s.tx_bytes();
+        ss.rxLatency = s.latency();
+        ss.rxJitter = s.jitter();
 
         aggrPort.rxPkts += ss.rxPkts;
         aggrPort.txPkts += ss.txPkts;
@@ -271,6 +289,11 @@ void StreamStatsModel::appendStreamStatsList(
         aggrGuid.txBytes += ss.txBytes;
         if (s.tx_duration() > aggrGuid.txDuration)
             aggrGuid.txDuration = s.tx_duration(); // XXX: use largest or avg?
+        if (ss.rxLatency) {
+            aggrGuid.latencySum += ss.rxLatency;
+            aggrGuid.jitterSum += ss.rxJitter;
+            aggrGuid.latencyCount++;
+        }
 
         aggrAggr.rxPkts += ss.rxPkts;
         aggrAggr.txPkts += ss.txPkts;
@@ -279,6 +302,11 @@ void StreamStatsModel::appendStreamStatsList(
         aggrAggr.txBytes += ss.txBytes;
         if (aggrGuid.txDuration > aggrAggr.txDuration)
             aggrAggr.txDuration = aggrGuid.txDuration;
+        if (ss.rxLatency) {
+            aggrAggr.latencySum += ss.rxLatency;
+            aggrAggr.jitterSum += ss.rxJitter;
+            aggrAggr.latencyCount++;
+        }
 
         if (!portList_.contains(pgp))
             portList_.append(pgp);
