@@ -24,10 +24,6 @@ along with this program.  If not, see <http://www.gnu.org/licenses/>
 
 #include <QMouseEvent>
 
-#if QT_VERSION >= 0x050000
-#error "Do we even need this anymore?"
-#endif
-
 class XTreeView : public QTreeView
 {
 public:
