@@ -134,6 +134,7 @@ QVariant IgmpProtocol::fieldData(int index, FieldAttrib attrib,
             case FieldName:            
                 return QString("Group Address");
             case FieldValue:
+                return grpIp;
             case FieldTextValue:
                 return QHostAddress(grpIp).toString();
             case FieldFrameValue:
@@ -289,8 +290,14 @@ bool IgmpProtocol::setFieldData(int index, const QVariant &value,
         }
         case kGroupAddress:
         {
+            quint32 ip = value.toUInt(&isOk);
+            if (isOk) {
+                data.mutable_group_address()->set_v4(ip);
+                break;
+            }
+
             QHostAddress addr(value.toString());
-            quint32 ip = addr.toIPv4Address();
+            ip = addr.toIPv4Address();
             isOk = (addr.protocol() == QAbstractSocket::IPv4Protocol);
             if (isOk)
                 data.mutable_group_address()->set_v4(ip);
@@ -306,6 +313,7 @@ bool IgmpProtocol::setFieldData(int index, const QVariant &value,
                 quint32 ip = QHostAddress(str).toIPv4Address();
                 data.add_sources()->set_v4(ip);
             }
+            isOk = true;
             break;
         }
 
@@ -332,6 +340,7 @@ bool IgmpProtocol::setFieldData(int index, const QVariant &value,
                             QHostAddress(src).toIPv4Address());
                 }
             }
+            isOk = true;
 
             break;
         }
@@ -347,18 +356,5 @@ _exit:
 
 quint16 IgmpProtocol::checksum(int streamIndex) const
 {
-    quint16 cks;
-    quint32 sum = 0;
-
-    // TODO: add as a new CksumType (CksumIgmp?) and implement in AbsProto 
-    cks = protocolFrameCksum(streamIndex, CksumIp);
-    sum += (quint16) ~cks;
-    cks = protocolFramePayloadCksum(streamIndex, CksumIp);
-    sum += (quint16) ~cks;
-    while (sum >> 16)
-        sum = (sum & 0xFFFF) + (sum >> 16);
-
-    cks = (~sum) & 0xFFFF;
-
-    return cks;
+    return AbstractProtocol::protocolFrameCksum(streamIndex, CksumIcmpIgmp);
 }
