@@ -232,24 +232,9 @@ int PayloadProtocol::protocolFrameVariableCount() const
     int count = AbstractProtocol::protocolFrameVariableCount();
 
     if (data.pattern_mode() == OstProto::Payload::e_dp_random)
-    {
-        switch(mpStream->sendUnit())
-        {
-        case OstProto::StreamControl::e_su_packets:
-            return mpStream->numPackets();
+        return mpStream->frameCount();
 
-        case OstProto::StreamControl::e_su_bursts:
-            return int(mpStream->numBursts() 
-                    * mpStream->burstSize() 
-                    * mpStream->burstRate());
-        }
-    }
-
-    if (mpStream->lenMode() != StreamBase::e_fl_fixed)
-    {
-        count = AbstractProtocol::lcm(count, 
-                mpStream->frameLenMax() - mpStream->frameLenMin() + 1);
-    }
+    count = AbstractProtocol::lcm(count, mpStream->frameSizeVariableCount());
 
     return count;
 }
