@@ -40,6 +40,7 @@ along with this program.  If not, see <http://www.gnu.org/licenses/>
 #include "ip6over4config.h"
 #include "ip6over6config.h"
 // L4 Protocol Widgets
+#include "greconfig.h"
 #include "icmpconfig.h"
 #include "igmpconfig.h"
 #include "mldconfig.h"
@@ -124,6 +125,9 @@ ProtocolWidgetFactory::ProtocolWidgetFactory()
             (void*) Ip6over6ConfigForm::createInstance);
 
     // Layer 4 Protocols
+    OstProtocolWidgetFactory->registerProtocolConfigWidget(
+            OstProto::Protocol::kGreFieldNumber,
+            (void*) GreConfigForm::createInstance);
     OstProtocolWidgetFactory->registerProtocolConfigWidget(
             OstProto::Protocol::kIcmpFieldNumber, 
             (void*) IcmpConfigForm::createInstance);
