@@ -21,6 +21,7 @@ along with this program.  If not, see <http://www.gnu.org/licenses/>
 
 #include "../common/abstractprotocol.h"
 #include "../common/framevalueattrib.h"
+#include "../common/packet.h"
 #include "../common/streambase.h"
 #include "devicemanager.h"
 #include "interfaceinfo.h"
@@ -54,6 +55,8 @@ AbstractPort::AbstractPort(int id, const char *device)
     maxStatsValue_ = ULLONG_MAX; // assume 64-bit stats
     memset((void*) &stats_, 0, sizeof(stats_));
     resetStats();
+
+    streamTiming_ = StreamTiming::instance();
 }
 
 AbstractPort::~AbstractPort()
@@ -103,6 +106,10 @@ bool AbstractPort::modify(const OstProto::Port &port)
     bool ret = true;
 
     //! \todo Use reflection to find out which fields are set
+    if (port.has_user_description()) {
+        data_.set_user_description(port.user_description());
+    }
+
     if (port.has_is_exclusive_control())
     {
         bool val = port.is_exclusive_control();
@@ -193,6 +200,13 @@ void AbstractPort::addNote(QString note)
 
 bool AbstractPort::setTrackStreamStats(bool enable)
 {
+    // XXX: This function is called by modify() in context of the RPC
+    // thread (1 thread per connected client), but the StreamTiming
+    // singleton resides in the main thread and its' start/stop methods
+    // start/stop timers which cannot be done across Qt Threads. Hence
+    // this slightly hacky way of invoking those methods
+    QMetaObject::invokeMethod(streamTiming_, enable ? "start" : "stop",
+        Qt::QueuedConnection, Q_ARG(uint, id()));
     data_.set_is_tracking_stream_stats(enable);
 
     return true;
@@ -228,6 +242,10 @@ int AbstractPort::updatePacketList()
 
 int AbstractPort::updatePacketListSequential()
 {
+    quint64 duration = 0;  // in nanosec
+    quint64 totalPkts = 0;
+    QList<uint> ttagMarkers;
+    uint ttagRepeatInterval;
     FrameValueAttrib packetListAttrib;
     long    sec = 0; 
     long    nsec = 0;
@@ -255,6 +273,8 @@ int AbstractPort::updatePacketListSequential()
             quint64 npy1 = 0, npy2 = 0;
             quint64 loopDelay;
             ulong frameVariableCount = streamList_[i]->frameVariableCount();
+            bool hasTtag = streamList_[i]->hasProtocol(
+                                OstProto::Protocol::kSignFieldNumber);
 
             // We derive n, x, y such that
             // n * x + y = total number of packets to be sent
@@ -322,11 +342,13 @@ int AbstractPort::updatePacketListSequential()
 
             if (n >= 1) {
                 loopNextPacketSet(x, n, 0, loopDelay);
-                qDebug("PacketSet: n = %lu, x = %lu", n, x);
+                qDebug("PacketSet: n = %lu, x = %lu, delay = %llu ns",
+                        n, x, loopDelay);
             }
             else if (n == 0)
                 x = 0;
 
+            quint64 pktCount = n*x + y;
             for (uint j = 0; j < (x+y); j++)
             {
                 
@@ -343,7 +365,8 @@ int AbstractPort::updatePacketListSequential()
                 // Create a packet set for 'y' with repeat = 1
                 if (j == x) {
                     loopNextPacketSet(y, 1, 0, loopDelay);
-                    qDebug("PacketSet: n = 1, y = %lu", y);
+                    qDebug("PacketSet: n = 1, y = %lu, delay = %llu",
+                            y, loopDelay);
                 }
 
                 qDebug("q(%d, %d) sec = %lu nsec = %lu",
@@ -379,6 +402,27 @@ int AbstractPort::updatePacketListSequential()
                 }
             }
 
+            // loopDelay == 0 implies 0 pps i.e. top speed
+            // For ttag calc/config below we need loopDelay to be non-zero,
+            // so we re-calc based on max line rate (speed). If we don't
+            // have the actual port speed, we assume 1000 Mbps
+            if (loopDelay == 0) {
+                double maxSpeed = data_.speed() ? data_.speed(): 1000;
+                double maxPktRate = (maxSpeed*1e6)
+                                        /(8*(streamList_[i]->frameLenAvg()
+                                                + Packet::kEthOverhead));
+                loopDelay = 1e9/maxPktRate; // in nanosec
+            }
+
+            // Add a Ttag marker after every kTtagTimeInterval_ worth of pkts
+            if (hasTtag) {
+                uint ttagPktInterval = kTtagTimeInterval_*1e9/loopDelay;
+                for (uint k = 0; k < pktCount; k += ttagPktInterval)
+                    ttagMarkers.append(totalPkts + k);
+            }
+            totalPkts += pktCount;
+            duration += pktCount*loopDelay; // in nanosecs
+
             switch(streamList_[i]->nextWhat())
             {
                 case StreamBase::e_nw_stop:
@@ -397,9 +441,10 @@ int AbstractPort::updatePacketListSequential()
                          returnToQIdx = 0;
                      */
 
-                    setPacketListLoopMode(true, 0, 
-                            streamList_[i]->sendUnit() == 
-                                StreamBase::e_su_bursts ? ibg1 : ipg1);
+                    // XXX: no list loop delay required since we don't create
+                    // any implicit packet sets now
+                    setPacketListLoopMode(true, 0, 0);
+                    qDebug("Seq mode list loop true with 0 delay");
                     goto _stop_no_more_pkts;
 
                 case StreamBase::e_nw_goto_next:
@@ -414,8 +459,17 @@ int AbstractPort::updatePacketListSequential()
         } // if (stream is enabled)
     } // for (numStreams)
 
-_out_of_memory:
 _stop_no_more_pkts:
+    // See comments in updatePacketListInterleaved() for calc explanation
+    ttagRepeatInterval = ttagMarkers.isEmpty() ? 0 :
+             qMax(uint(kTtagTimeInterval_*1e9/(duration)), 1U)
+                * totalPkts;
+    if (!setPacketListTtagMarkers(ttagMarkers, ttagRepeatInterval)) {
+        clearPacketList(); // don't leave it half baked/inconsitent
+        packetListAttrib.errorFlags |= FrameValueAttrib::OutOfMemoryError;
+    }
+
+_out_of_memory:
     isSendQueueDirty_ = false;
 
     qDebug("PacketListAttrib = %x",
@@ -428,7 +482,9 @@ int AbstractPort::updatePacketListInterleaved()
     FrameValueAttrib packetListAttrib;
     int numStreams = 0;
     quint64 minGap = ULLONG_MAX;
-    quint64 duration = quint64(1e3);
+    quint64 duration = quint64(1e3); // 1000ns (1us)
+
+    // TODO: convert the below to a QVector of struct aggregating all list vars
     QList<int> streamId;
     QList<quint64> ibg1, ibg2;
     QList<quint64> nb1, nb2;
@@ -438,6 +494,7 @@ int AbstractPort::updatePacketListInterleaved()
     QList<ulong> pktCount, burstCount;
     QList<ulong> burstSize;
     QList<bool> isVariable;
+    QList<bool> hasTtag;
     QList<QByteArray> pktBuf;
     QList<ulong> pktLen;
     int activeStreamCount = 0;
@@ -461,6 +518,9 @@ int AbstractPort::updatePacketListInterleaved()
     // First sort the streams by ordinalValue
     std::sort(streamList_.begin(), streamList_.end(), StreamBase::StreamLessThan);
 
+    // FIXME: we are calculating n[bp][12], i[bp]g[12] for a duration of 1sec;
+    // this was fine when the actual packet list duration was also 1sec. But
+    // in the current code (post Turbo changes), the latter can be different!
     for (int i = 0; i < streamList_.size(); i++)
     {
         if (!streamList_[i]->isEnabled())
@@ -584,6 +644,8 @@ int AbstractPort::updatePacketListInterleaved()
             packetListAttrib += attrib;
         }
 
+        hasTtag.append(streamList_[i]->hasProtocol(
+                            OstProto::Protocol::kSignFieldNumber));
         numStreams++;
     } // for i
 
@@ -591,6 +653,7 @@ int AbstractPort::updatePacketListInterleaved()
     // i.e. send all streams "simultaneously" as fast as possible
     // as a result all streams will be at the same rate e.g. for 2 streams,
     // it would 50% each; for 3 streams - all at 33.3% and so on
+    // FIXME: Should we calc minGap based on max line rate and avg pkt size?
     if (minGap == ULLONG_MAX) {
         minGap = 1;
         duration = 1;
@@ -606,12 +669,99 @@ int AbstractPort::updatePacketListInterleaved()
 
     uchar* buf;
     int len;
-    quint64 durSec = duration/ulong(1e9);
-    quint64 durNsec = duration % ulong(1e9);
+    const quint64 durSec = duration/ulong(1e9);
+    const quint64 durNsec = duration % ulong(1e9);
     quint64 sec = 0; 
     quint64 nsec = 0;
     quint64 lastPktTxSec = 0;
     quint64 lastPktTxNsec = 0;
+
+    // Count total packets we are going to add, so that we can create
+    // an explicit packet set first
+    // TODO: Find less expensive way to do this counting
+    quint64 totalPkts = 0;
+    QVector<ulong> ttagSchedSec(numStreams, 0);
+    QVector<ulong> ttagSchedNsec(numStreams, 0);
+    QList<uint> ttagMarkers;
+    uint ttagRepeatInterval;
+
+    do
+    {
+        for (int i = 0; i < numStreams; i++)
+        {
+            // If a packet is not scheduled yet, look at the next stream
+            if ((schedSec.at(i) > sec) || (schedNsec.at(i) > nsec))
+                continue;
+
+            // Ttag marker every TtagTimeInterval for each stream
+            if (hasTtag.at(i)
+                    && ((schedSec.at(i) > ttagSchedSec.at(i))
+                            || ((schedSec.at(i) == ttagSchedSec.at(i))
+                                && (schedNsec.at(i) >= ttagSchedNsec.at(i))))) {
+                ttagMarkers.append(totalPkts);
+                ttagSchedSec[i] = schedSec.at(i) + kTtagTimeInterval_;
+                ttagSchedNsec[i] = schedNsec.at(i);
+            }
+
+            for (uint j = 0; j < burstSize[i]; j++)
+            {
+                pktCount[i]++;
+                schedNsec[i] += (pktCount.at(i) < np1.at(i)) ?
+                    ipg1.at(i) : ipg2.at(i);
+                while (schedNsec.at(i) >= 1e9)
+                {
+                    schedSec[i]++;
+                    schedNsec[i] -= long(1e9);
+                }
+                lastPktTxSec = sec;
+                lastPktTxNsec = nsec;
+                totalPkts++;
+            }
+
+            burstCount[i]++;
+            schedNsec[i] += (burstCount.at(i) < nb1.at(i)) ?
+                    ibg1.at(i) : ibg2.at(i);
+            while (schedNsec.at(i) >= 1e9)
+            {
+                schedSec[i]++;
+                schedNsec[i] -= long(1e9);
+            }
+        }
+
+        nsec += minGap;
+        while (nsec >= 1e9)
+        {
+            sec++;
+            nsec -= long(1e9);
+        }
+    } while ((sec < durSec) || ((sec == durSec) && (nsec < durNsec)));
+
+    qint64 delaySec = durSec - lastPktTxSec;
+    qint64 delayNsec = durNsec - lastPktTxNsec;
+    while (delayNsec < 0)
+    {
+        delayNsec += long(1e9);
+        delaySec--;
+    }
+
+    // XXX: For interleaved mode, we ALWAYS have a single packet set with
+    // one repeat
+    loopNextPacketSet(totalPkts, 1, delaySec, delayNsec);
+    qDebug("Interleaved single PacketSet of size %lld, duration %llu.%09llu "
+           "repeat 1 and delay %lld.%09lld",
+            totalPkts, durSec, durNsec, delaySec, delayNsec);
+
+    // Reset working sched/counts before building the packet list
+    sec = nsec = 0;
+    for (int i = 0; i < numStreams; i++)
+    {
+        schedSec[i] = 0;
+        schedNsec[i] = 0;
+        pktCount[i] = 0;
+        burstCount[i] = 0;
+    }
+
+    // Now build the packet list
     do
     {
         for (int i = 0; i < numStreams; i++)
@@ -639,14 +789,12 @@ int AbstractPort::updatePacketListInterleaved()
                 if (len <= 0)
                     continue;
 
-                qDebug("q(%d) sec = %llu nsec = %llu", i, sec, nsec);
+                qDebug("q(%d) TS = %llu.%09llu", i, sec, nsec);
                 if (!appendToPacketList(sec, nsec, buf, len)) {
                     clearPacketList(); // don't leave it half baked/inconsitent
                     packetListAttrib.errorFlags |= FrameValueAttrib::OutOfMemoryError;
                     goto _out_of_memory;
                 }
-                lastPktTxSec = sec;
-                lastPktTxNsec = nsec;
 
                 pktCount[i]++;
                 schedNsec[i] += (pktCount.at(i) < np1.at(i)) ? 
@@ -674,18 +822,25 @@ int AbstractPort::updatePacketListInterleaved()
             sec++;
             nsec -= long(1e9);
         }
-    } while ((sec < durSec) || (nsec < durNsec));
+    } while ((sec < durSec) || ((sec == durSec) && (nsec < durNsec)));
 
-    {
-        qint64 delaySec = durSec - lastPktTxSec;
-        qint64 delayNsec = durNsec - lastPktTxNsec;
-        while (delayNsec < 0)
-        {
-            delayNsec += long(1e9);
-            delaySec--;
-        }
-        qDebug("loop Delay = %lld/%lld", delaySec, delayNsec);
-        setPacketListLoopMode(true, delaySec, delayNsec);
+    // XXX: The single packet has the delay, so no list loop delay required
+    // XXX: Both seq/interleaved mode no longer use list loop delay!
+    setPacketListLoopMode(true, 0, 0);
+
+    // XXX: TTag repeat interval calculation:
+    // CASE 1. pktListDuration < kTtagTimeInterval:
+    //      e.g. if pktListDuration is 1sec and TtagTimerInterval is 5s, we
+    //      skip 5 times total packets before we repeat the markers
+    // CASE 2. pktListDuration > kTtagTimeInterval:
+    //      e.g. if pktListDuration is 7sec and TtagTimerInterval is 5s, we
+    //      skip repeat markers every pktList iteration
+    ttagRepeatInterval = ttagMarkers.isEmpty() ? 0 :
+                 qMax(uint(kTtagTimeInterval_*1e9/(durSec*1e9+durNsec)), 1U)
+                     * totalPkts;
+    if (!setPacketListTtagMarkers(ttagMarkers, ttagRepeatInterval)) {
+        clearPacketList(); // don't leave it half baked/inconsitent
+        packetListAttrib.errorFlags |= FrameValueAttrib::OutOfMemoryError;
     }
 
 _out_of_memory:
@@ -730,20 +885,37 @@ void AbstractPort::stats(PortStats *stats)
                         stats_.rxFrameErrors + (maxStatsValue_ - epochStats_.rxFrameErrors);
 }
 
+StreamTiming::Stats AbstractPort::streamTimingStats(uint guid)
+{
+    return streamTiming_->stats(id(), guid);
+}
+
+void AbstractPort::clearStreamTiming(uint guid)
+{
+    streamTiming_->clear(id(), guid);
+}
+
 void AbstractPort::streamStats(uint guid, OstProto::StreamStatsList *stats)
 {
     // In case stats are being maintained elsewhere
     updateStreamStats();
 
+    // Lock for read here as updateStreamStats() above will take write lock
+    // and the lock is NOT recursive
+    QReadLocker lock(&streamStatsLock_);
+
     if (streamStats_.contains(guid))
     {
         StreamStatsTuple sst = streamStats_.value(guid);
         OstProto::StreamStats *s = stats->add_stream_stats();
+        StreamTiming::Stats t = streamTimingStats(guid);
 
         s->mutable_stream_guid()->set_id(guid);
         s->mutable_port_id()->set_id(id());
 
         s->set_tx_duration(lastTransmitDuration());
+        s->set_latency(t.latency);
+        s->set_jitter(t.jitter);
 
         s->set_tx_pkts(sst.tx_pkts);
         s->set_tx_bytes(sst.tx_bytes);
@@ -757,6 +929,10 @@ void AbstractPort::streamStatsAll(OstProto::StreamStatsList *stats)
     // In case stats are being maintained elsewhere
     updateStreamStats();
 
+    // Lock for read here as updateStreamStats() above will take write lock
+    // and the lock is NOT recursive
+    QReadLocker lock(&streamStatsLock_);
+
     // FIXME: change input param to a non-OstProto type and/or have
     // a getFirst/Next like API?
     double txDur = lastTransmitDuration();
@@ -766,11 +942,14 @@ void AbstractPort::streamStatsAll(OstProto::StreamStatsList *stats)
         i.next();
         StreamStatsTuple sst = i.value();
         OstProto::StreamStats *s = stats->add_stream_stats();
+        StreamTiming::Stats t = streamTimingStats(i.key());
 
         s->mutable_stream_guid()->set_id(i.key());
         s->mutable_port_id()->set_id(id());
 
         s->set_tx_duration(txDur);
+        s->set_latency(t.latency);
+        s->set_jitter(t.jitter);
 
         s->set_tx_pkts(sst.tx_pkts);
         s->set_tx_bytes(sst.tx_bytes);
@@ -781,12 +960,16 @@ void AbstractPort::streamStatsAll(OstProto::StreamStatsList *stats)
 
 void AbstractPort::resetStreamStats(uint guid)
 {
+    QWriteLocker lock(&streamStatsLock_);
     streamStats_.remove(guid);
+    clearStreamTiming(guid);
 }
 
 void AbstractPort::resetStreamStatsAll()
 {
+    QWriteLocker lock(&streamStatsLock_);
     streamStats_.clear();
+    clearStreamTiming();
 }
 
 void AbstractPort::clearDeviceNeighbors()
