@@ -20,18 +20,21 @@ along with this program.  If not, see <http://www.gnu.org/licenses/>
 #ifndef _SERVER_ABSTRACT_PORT_H
 #define _SERVER_ABSTRACT_PORT_H
 
+#include "../common/protocol.pb.h"
 #include "streamstats.h"
+#include "streamtiming.h"
 
 #include <QList>
+#include <QReadWriteLock>
 #include <QtGlobal>
 
-#include "../common/protocol.pb.h"
+#include <limits.h>
 
 class DeviceManager;
 struct InterfaceInfo;
-class StreamBase;
 class PacketBuffer;
 class QIODevice;
+class StreamBase;
 
 // TODO: send notification back to client(s)
 #define Xnotify qWarning
@@ -105,6 +108,8 @@ public:
             int length) = 0;
     virtual void setPacketListLoopMode(bool loop, 
             quint64 secDelay, quint64 nsecDelay) = 0;
+    virtual bool setPacketListTtagMarkers(QList<uint> markers,
+            uint repeatInterval) = 0;
     int updatePacketList();
 
     virtual void startTransmit() = 0;
@@ -120,6 +125,9 @@ public:
     void stats(PortStats *stats);
     void resetStats() { epochStats_ = stats_; }
 
+    StreamTiming::Stats streamTimingStats(uint guid);
+    void clearStreamTiming(uint guid = UINT_MAX);
+
     // FIXME: combine single and All calls?
     void streamStats(uint guid, OstProto::StreamStatsList *stats);
     void streamStatsAll(OstProto::StreamStatsList *stats);
@@ -156,8 +164,11 @@ protected:
     quint64 maxStatsValue_;
     struct PortStats    stats_;
     StreamStats streamStats_;
+    QReadWriteLock streamStatsLock_;
     //! \todo Need lock for stats access/update
 
+    const uint kTtagTimeInterval_{5}; // in seconds
+
     struct InterfaceInfo *interfaceInfo_;
     DeviceManager *deviceManager_;
 
@@ -177,6 +188,8 @@ private:
     QList<StreamBase*>  streamList_;
 
     struct PortStats    epochStats_;
+
+    StreamTiming *streamTiming_{nullptr};
 };
 
 #endif
