@@ -222,6 +222,7 @@ void DeviceManager::getDeviceList(
 
 void DeviceManager::receivePacket(PacketBuffer *pktBuf)
 {
+    QMutexLocker locker(&listLock_);
     uchar *pktData = pktBuf->data();
     int offset = 0;
     EmulDevice dk(this);
@@ -267,7 +268,8 @@ _eth_type:
         vlan = qFromBigEndian<quint16>(pktData + offset);
         dk.setVlan(idx++, vlan);
         offset += 2;
-        qDebug("%s: idx: %d vlan %d", __FUNCTION__, idx, vlan);
+        qDebug("%s: idx: %d vlan: 0x%04x/%d", __FUNCTION__,
+                idx, vlan, vlan & 0x0fff);
         goto _eth_type;
     }
 
@@ -382,7 +384,8 @@ _eth_type:
         vlan = qFromBigEndian<quint16>(pktData + offset);
         dk.setVlan(idx++, vlan);
         offset += 2;
-        qDebug("%s: idx: %d vlan %d", __FUNCTION__, idx, vlan);
+        qDebug("%s: idx: %d vlan: 0x%04x/%d", __FUNCTION__,
+                idx, vlan, vlan & 0x0fff);
         goto _eth_type;
     }
 
@@ -401,6 +404,7 @@ void DeviceManager::enumerateDevices(
     const OstProto::DeviceGroup *deviceGroup,
     Operation oper)
 {
+    QMutexLocker locker(&listLock_);
     EmulDevice dk(this);
     OstEmul::VlanEmulation pbVlan = deviceGroup->encap()
                                         .GetExtension(OstEmul::vlan);
