@@ -27,6 +27,7 @@ along with this program.  If not, see <http://www.gnu.org/licenses/>
 #include "abstractport.h"
 #include "pcapextra.h"
 #include "pcaprxstats.h"
+#include "pcaptxttagstats.h"
 #include "pcapsession.h"
 #include "pcaptransmitter.h"
 
@@ -47,6 +48,7 @@ public:
     virtual void clearPacketList() { 
         transmitter_->clearPacketList();
         setPacketListLoopMode(false, 0, 0);
+        setPacketListTtagMarkers(QList<uint>(), 0);
     }
     virtual void loopNextPacketSet(qint64 size, qint64 repeats,
             long repeatDelaySec, long repeatDelayNsec) {
@@ -61,6 +63,11 @@ public:
     {
         transmitter_->setPacketListLoopMode(loop, secDelay, nsecDelay);
     }
+    virtual bool setPacketListTtagMarkers(QList<uint> markers,
+            uint repeatInterval)
+    {
+        return transmitter_->setPacketListTtagMarkers(markers, repeatInterval);
+    }
 
     virtual void startTransmit() { 
         Q_ASSERT(!isDirty());
@@ -77,6 +84,8 @@ public:
     virtual bool isCaptureOn()  { return capturer_->isRunning(); }
     virtual QIODevice* captureData() { return capturer_->captureFile(); }
 
+    virtual void updateStreamStats();
+
     virtual void startDeviceEmulation();
     virtual void stopDeviceEmulation();
     virtual int sendEmulationPacket(PacketBuffer *pktBuf);
@@ -132,7 +141,6 @@ protected:
         QString         device_;
         volatile bool   stop_;
         QTemporaryFile  capFile_;
-        pcap_t          *handle_;
         pcap_dumper_t   *dumpHandle_;
         volatile State  state_;
     };
@@ -159,13 +167,14 @@ protected:
         QString         device_;
         DeviceManager   *deviceManager_;
         volatile bool   stop_;
-        pcap_t          *handle_;
         volatile State  state_;
     };
 
     PortMonitor     *monitorRx_;
     PortMonitor     *monitorTx_;
 
+    PcapRxStats *rxStatsPoller_;
+
     void updateNotes();
 
 private:
@@ -175,7 +184,7 @@ private:
     PcapTransmitter *transmitter_;
     PortCapturer    *capturer_;
     EmulationTransceiver *emulXcvr_;
-    PcapRxStats *rxStatsPoller_;
+    PcapTxTtagStats *txTtagStatsPoller_;
 
     static pcap_if_t *deviceList_;
 };
