@@ -20,19 +20,26 @@ along with this program.  If not, see <http://www.gnu.org/licenses/>
 #include "pcaprxstats.h"
 
 #include "pcapextra.h"
+#include "../common/debugdefs.h"
 #include "../common/sign.h"
+#include "settings.h"
+#include "streamtiming.h"
 
 #define Xnotify qWarning // FIXME
 
-PcapRxStats::PcapRxStats(const char *device, StreamStats &portStreamStats)
-    : streamStats_(portStreamStats)
+PcapRxStats::PcapRxStats(const char *device, int id)
 {
+    setObjectName(QString("Rx$:%1").arg(device));
     device_ = QString::fromLatin1(device);
     stop_ = false;
     state_ = kNotStarted;
     isDirectional_ = true;
 
     handle_ = NULL;
+
+    portId_ = id;
+
+    timing_ = StreamTiming::instance();
 }
 
 pcap_t* PcapRxStats::handle()
@@ -50,6 +57,20 @@ void PcapRxStats::run()
             SignProtocol::magic(), 0, BASE_HEX);
     // XXX: Exclude ICMP packets which contain an embedded signed packet
     //      For now we check upto 4 vlan tags
+    // XXX: libpcap for Linux has a special bpf vlan check which generates
+    //      incorrect BPF instructions for our capture filter expression,
+    //      so we modify it to work correctly
+    //      See https://srivatsp.com/ostinato/ostinato-rx-stream-stats-zero/
+#ifdef Q_OS_LINUX
+    capture_filter.prepend(
+        "not ("
+            "icmp or "
+            "(vlan and icmp) or "
+            "(vlan and icmp) or "
+            "(vlan and icmp) or "
+            "(vlan and icmp) "
+        ") and ");
+#else
     capture_filter.append(
         "and not ("
             "icmp or "
@@ -58,8 +79,15 @@ void PcapRxStats::run()
             "(vlan and icmp) or "
             "(vlan and icmp) "
         ")");
+#endif
+
+    // Override filter expression if one is specified in .ini
+    if (appSettings->contains(kInternalRxStatsFilterKey))
+        capture_filter = appSettings->value(kInternalRxStatsFilterKey)
+                            .toString();
 
     qDebug("In %s", __PRETTY_FUNCTION__);
+    qDebug("RxStats Filter: %s", qPrintable(capture_filter));
 
     handle_ = pcap_open_live(qPrintable(device_), 65535,
                     flags, 100 /* ms */, errbuf);
@@ -104,6 +132,7 @@ void PcapRxStats::run()
     }
 
 _skip_filter:
+    clearDebugStats();
     PcapSession::preRun();
     state_ = kRunning;
     while (1) {
@@ -114,8 +143,25 @@ _skip_filter:
         ret = pcap_next_ex(handle_, &hdr, &data);
         switch (ret) {
             case 1: {
-                uint guid;
-                if (SignProtocol::packetGuid(data, hdr->caplen, &guid)) {
+                uint ttagId, guid;
+#ifdef Q_OS_WIN32
+                // Npcap (Windows) doesn't support direction, so packets
+                // Tx by PcapTxThread are received back by us here - use
+                // TxPort to filter out. TxPort is returned as byte 1 of
+                // ttagId (byte 0 is ttagId).
+                // If TxPort is us ==> Tx Packet, so skip
+                // FIXME: remove once npcap supports pcap direction
+                if (SignProtocol::packetTtagId(data, hdr->caplen, &ttagId, &guid)
+                        && (ttagId >> 8 != uint(portId_))) {
+                    ttagId &= 0xFF;
+                    timing_->recordRxTime(portId_, guid, ttagId, hdr->ts);
+                }
+#else
+                if (SignProtocol::packetTtagId(data, hdr->caplen, &ttagId, &guid)) {
+                    timing_->recordRxTime(portId_, guid, ttagId, hdr->ts);
+                }
+#endif
+                if (guid != SignProtocol::kInvalidGuid) {
                     streamStats_[guid].rx_pkts++;
                     streamStats_[guid].rx_bytes += hdr->caplen;
                 }
@@ -143,7 +189,6 @@ _skip_filter:
         }
     }
     PcapSession::postRun();
-
     pcap_close(handle_);
     handle_ = NULL;
     stop_ = false;
@@ -172,7 +217,7 @@ bool PcapRxStats::stop()
 {
     if (state_ == kRunning) {
         stop_ = true;
-        PcapSession::stop(handle_);
+        PcapSession::stop();
         while (state_ == kRunning)
             QThread::msleep(10);
     }
@@ -191,3 +236,21 @@ bool PcapRxStats::isDirectional()
 {
     return isDirectional_;
 }
+
+// XXX: Stats are reset on read
+void PcapRxStats::updateRxStreamStats(StreamStats &streamStats)
+{
+    QMutexLocker lock(&streamStatsLock_);
+    StreamStatsIterator i(streamStats_);
+
+    while (i.hasNext())
+    {
+        i.next();
+        uint guid = i.key();
+        StreamStatsTuple sst = i.value();
+
+        streamStats[guid].rx_pkts += sst.rx_pkts;
+        streamStats[guid].rx_bytes += sst.rx_bytes;
+    }
+    streamStats_.clear();
+}
