@@ -24,10 +24,14 @@ along with this program.  If not, see <http://www.gnu.org/licenses/>
 
 #include "pcapsession.h"
 
+#include <QMutex>
+
+class StreamTiming;
+
 class PcapRxStats: public PcapSession
 {
 public:
-    PcapRxStats(const char *device, StreamStats &portStreamStats);
+    PcapRxStats(const char *device, int id);
     pcap_t* handle();
     void run();
     bool start();
@@ -35,6 +39,7 @@ public:
     bool isRunning();
     bool isDirectional();
 
+    void updateRxStreamStats(StreamStats &streamStats); // Reset on read
 private:
     enum State {
         kNotStarted,
@@ -43,11 +48,15 @@ private:
     };
 
     QString device_;
-    StreamStats &streamStats_;
+    StreamStats streamStats_;
+    QMutex streamStatsLock_;
     volatile bool stop_;
-    pcap_t *handle_;
     volatile State state_;
     bool isDirectional_;
+
+    int portId_;
+
+    StreamTiming *timing_{nullptr};
 };
 
 #endif
