@@ -24,7 +24,7 @@ along with this program.  If not, see <http://www.gnu.org/licenses/>
 
 #include <QThread>
 
-class StatsTuple;
+struct StatsTuple;
 
 class PcapTxStats : public QThread
 {
