@@ -19,9 +19,12 @@ along with this program.  If not, see <http://www.gnu.org/licenses/>
 
 #include "pcaptxthread.h"
 
+#include "sign.h"
 #include "statstuple.h"
 #include "timestamp.h"
 
+#include <QtDebug>
+
 PcapTxThread::PcapTxThread(const char *device)
 {
     char errbuf[PCAP_ERRBUF_SIZE] = "";
@@ -109,11 +112,6 @@ void PcapTxThread::clearPacketList()
 void PcapTxThread::loopNextPacketSet(qint64 size, qint64 repeats,
         long repeatDelaySec, long repeatDelayNsec)
 {
-    // Since we create implicit packetset for this case, skip
-    // This case => Packet set for y when x = 0 or n==1 in n*x+y
-    if (repeats == 1)
-        return;
-
     currentPacketSequence_ = new PacketSequence(trackStreamStats_);
     currentPacketSequence_->repeatCount_ = repeats;
     currentPacketSequence_->usecDelay_ = repeatDelaySec * long(1e6)
@@ -136,24 +134,20 @@ bool PcapTxThread::appendToPacketList(long sec, long nsec,
     pktHdr.ts.tv_sec = sec;
     pktHdr.ts.tv_usec = nsec/1000;
 
-    if (currentPacketSequence_ == NULL ||
-            !currentPacketSequence_->hasFreeSpace(2*sizeof(pcap_pkthdr)+length))
-    {
-        if (currentPacketSequence_ != NULL)
-        {
-            long usecs;
+    // loopNextPacketSet should have created a seq
+    Q_ASSERT(currentPacketSequence_ != NULL);
 
-            usecs = (pktHdr.ts.tv_sec
-                        - currentPacketSequence_->lastPacket_->ts.tv_sec)
-                            * long(1e6);
-            usecs += (pktHdr.ts.tv_usec
-                        - currentPacketSequence_->lastPacket_->ts.tv_usec);
-            currentPacketSequence_->usecDelay_ = usecs;
-        }
+    // If not enough space, update usecDelay and alloc a new seq
+    if (!currentPacketSequence_->hasFreeSpace(2*sizeof(pcap_pkthdr)+length))
+    {
+        struct timeval diff;
+        timersub(&pktHdr.ts, &currentPacketSequence_->lastPacket_->ts, &diff);
+        currentPacketSequence_->usecDelay_ = diff.tv_usec;
+        if (diff.tv_sec)
+            currentPacketSequence_->usecDelay_ += diff.tv_sec*1e6;
 
         //! \todo (LOW): calculate sendqueue size
         currentPacketSequence_ = new PacketSequence(trackStreamStats_);
-
         packetSequenceList_.append(currentPacketSequence_);
 
         // Validate that the pkt will fit inside the new currentSendQueue_
@@ -169,6 +163,8 @@ bool PcapTxThread::appendToPacketList(long sec, long nsec,
     packetCount_++;
     packetListSize_ += repeatSize_ ?
                                 currentPacketSequence_->repeatCount_ : 1;
+
+    // Last packet of packet-set?
     if (repeatSize_ > 0 && packetCount_ == repeatSize_)
     {
         qDebug("repeatSequenceStart_=%d, repeatSize_ = %llu",
@@ -190,7 +186,7 @@ bool PcapTxThread::appendToPacketList(long sec, long nsec,
 
         repeatSize_ = 0;
 
-        // End current pktSeq and trigger a new pktSeq allocation for next pkt
+        // End current pktSeq
         currentPacketSequence_ = NULL;
     }
 
@@ -206,6 +202,27 @@ void PcapTxThread::setPacketListLoopMode(
     loopDelay_ = secDelay*long(1e6) + nsecDelay/1000;
 }
 
+bool PcapTxThread::setPacketListTtagMarkers(
+    QList<uint> markers,
+    uint repeatInterval)
+{
+    // XXX: Empty markers => no streams have Ttag
+    firstTtagPkt_ = markers.isEmpty() ? -1 : int(markers.first());
+
+    // Calculate delta markers
+    ttagDeltaMarkers_.clear();
+    for (int i = 1; i < markers.size(); i++)
+        ttagDeltaMarkers_.append(markers.at(i) - markers.at(i-1));
+    if (!markers.isEmpty()) {
+        ttagDeltaMarkers_.append(repeatInterval - markers.last()
+                                    + markers.first());
+        qDebug() << "TtagRepeatInterval:" << repeatInterval;
+        qDebug() << "FirstTtagPkt:" << firstTtagPkt_;
+        qDebug() << "TtagMarkers:" << ttagDeltaMarkers_;
+    }
+    return true;
+}
+
 void PcapTxThread::setHandle(pcap_t *handle)
 {
     if (usingInternalHandle_)
@@ -219,14 +236,20 @@ void PcapTxThread::setStats(StatsTuple *stats)
     stats_ = stats;
 }
 
-const StreamStats& PcapTxThread::streamStats()
+StreamStats PcapTxThread::streamStats()
 {
-    return streamStats_;
-}
+    // This function is typically called in client-specific-RPC-thread
+    // context; hence different client RPC threads may call this function,
+    // so use a lock. Although RPCs are protected by the portLock just
+    // for this purpose, the streamStats RPC takes a Read lock, so it can
+    // still happen that multiple RPC threads land up here - that's why
+    // this lock is required
+    QMutexLocker lock(&streamStatsLock_);
 
-void PcapTxThread::clearStreamStats()
-{
-    streamStats_.clear();
+    StreamStats ss(streamStats_); // Make a copy
+    streamStats_.clear();         // Reset on read semantics
+
+    return ss; // Return copy
 }
 
 void PcapTxThread::run()
@@ -261,39 +284,47 @@ void PcapTxThread::run()
                 packetSequenceList_.at(i)->repeatCount_,
                 packetSequenceList_.at(i)->repeatSize_,
                 packetSequenceList_.at(i)->usecDelay_);
-        qDebug("sendQ[%d]: pkts = %ld, usecDuration = %ld", i,
+        qDebug("sendQ[%d]: pkts = %ld, usecDuration = %ld, ttagL4CksumOfs = %hu", i,
                 packetSequenceList_.at(i)->packets_,
-                packetSequenceList_.at(i)->usecDuration_);
+                packetSequenceList_.at(i)->usecDuration_,
+                packetSequenceList_.at(i)->ttagL4CksumOffset_);
     }
 
+    qDebug() << "Loop:" << (returnToQIdx_ >= 0)
+             << "LoopDelay:" << loopDelay_;
+    qDebug() << "First Ttag: " << firstTtagPkt_
+             << "Ttag Markers:" << ttagDeltaMarkers_;
+
     lastStats_ = *stats_; // used for stream stats
 
+    // Init Ttag related vars. If no packets need ttag, firstTtagPkt_ is -1,
+    // so nextTagPkt_ is set to practically unreachable value (due to
+    // 64 bit counter wraparound time!)
+    ttagMarkerIndex_ = 0;
+    nextTtagPkt_ = stats_->pkts + firstTtagPkt_;
+
     getTimeStamp(&startTime);
     state_ = kRunning;
     i = 0;
-    while (i < packetSequenceList_.size())
-    {
-
+    while (i < packetSequenceList_.size()) {
 _restart:
         int rptSz  = packetSequenceList_.at(i)->repeatSize_;
         int rptCnt = packetSequenceList_.at(i)->repeatCount_;
 
-        for (int j = 0; j < rptCnt; j++)
-        {
-            for (int k = 0; k < rptSz; k++)
-            {
+        for (int j = 0; j < rptCnt; j++) {
+            for (int k = 0; k < rptSz; k++) {
                 int ret;
                 PacketSequence *seq = packetSequenceList_.at(i+k);
 #ifdef Q_OS_WIN32
                 TimeStamp ovrStart, ovrEnd;
 
-                if (seq->usecDuration_ <= long(1e6)) // 1s
-                {
+                // Use Windows-only pcap_sendqueue_transmit() if duration < 1s
+                // and no stream timing is configured
+                if (seq->usecDuration_ <= long(1e6) && firstTtagPkt_ < 0) {
                     getTimeStamp(&ovrStart);
                     ret = pcap_sendqueue_transmit(handle_,
                             seq->sendQueue_, kSyncTransmit);
-                    if (ret >= 0)
-                    {
+                    if (ret >= 0) {
                         stats_->pkts += seq->packets_;
                         stats_->bytes += seq->bytes_;
 
@@ -304,52 +335,42 @@ _restart:
                     }
                     if (stop_)
                         ret = -2;
-                }
-                else
-                {
-                    ret = sendQueueTransmit(handle_, seq->sendQueue_,
+                } else {
+                    ret = sendQueueTransmit(handle_, seq,
                             overHead, kSyncTransmit);
                 }
 #else
-                ret = sendQueueTransmit(handle_, seq->sendQueue_,
+                ret = sendQueueTransmit(handle_, seq,
                             overHead, kSyncTransmit);
 #endif
 
-                if (ret >= 0)
-                {
+                if (ret >= 0) {
                     long usecs = seq->usecDelay_ + overHead;
-                    if (usecs > 0)
-                    {
+                    if (usecs > 0) {
                         (*udelayFn_)(usecs);
                         overHead = 0;
-                    }
-                    else
+                    } else
                         overHead = usecs;
-                }
-                else
-                {
+                } else {
                     qDebug("error %d in sendQueueTransmit()", ret);
                     qDebug("overHead = %ld", overHead);
                     stop_ = false;
                     goto _exit;
                 }
-            }
-        }
+            } // rptSz
+        } // rptCnt
 
         // Move to the next Packet Set
         i += rptSz;
     }
 
-    if (returnToQIdx_ >= 0)
-    {
+    if (returnToQIdx_ >= 0) {
         long usecs = loopDelay_ + overHead;
 
-        if (usecs > 0)
-        {
+        if (usecs > 0) {
             (*udelayFn_)(usecs);
             overHead = 0;
-        }
-        else
+        } else
             overHead = usecs;
 
         i = returnToQIdx_;
@@ -365,7 +386,7 @@ _exit2:
     //Q_ASSERT(lastTxDuration_ >= 0);
 
     if (trackStreamStats_)
-        updateStreamStats();
+        updateTxStreamStats();
 
     state_ = kFinished;
 }
@@ -409,38 +430,77 @@ double PcapTxThread::lastTxDuration()
     return lastTxDuration_;
 }
 
-int PcapTxThread::sendQueueTransmit(pcap_t *p,
-        pcap_send_queue *queue, long &overHead, int sync)
+int PcapTxThread::sendQueueTransmit(pcap_t *p, PacketSequence *seq,
+        long &overHead, int sync)
 {
     TimeStamp ovrStart, ovrEnd;
     struct timeval ts;
+    pcap_send_queue *queue = seq->sendQueue_;
     struct pcap_pkthdr *hdr = (struct pcap_pkthdr*) queue->buffer;
     char *end = queue->buffer + queue->len;
 
     ts = hdr->ts;
-
     getTimeStamp(&ovrStart);
-    while((char*) hdr < end)
-    {
+    while((char*) hdr < end) {
         uchar *pkt = (uchar*)hdr + sizeof(*hdr);
         int pktLen = hdr->caplen;
+        bool ttagPkt = false;
+#if 0
+        quint16 origCksum = 0;
+#endif
 
-        if (sync)
-        {
+        // Time for a T-Tag packet?
+        if (stats_->pkts == nextTtagPkt_) {
+            ttagPkt = true;
+            // XXX: write 2xBytes instead of 1xHalf-word to avoid
+            // potential alignment problem
+            *(pkt+pktLen-5) = SignProtocol::kTypeLenTtag;
+            *(pkt+pktLen-6) = ttagId_;
+
+#if 0
+            // Recalc L4 checksum; use incremental checksum as per RFC 1624
+            // HC' = ~(~HC + ~m + m')
+            if (seq->ttagL4CksumOffset_) {
+                quint16 *cksum = reinterpret_cast<quint16*>(
+                                        pkt + seq->ttagL4CksumOffset_);
+                origCksum = qFromBigEndian<quint16>(*cksum);
+                // XXX: SignProtocol trailer
+                //      ... | <guid> | 0x61 |     0x00 | 0x22 | 0x1d10c0da
+                //      ... | <guid> | 0x61 | <TtagId> | 0x23 | 0x1d10c0da
+                // For odd pkt Length, Ttag spans across 2 half-words
+                // XXX: Hardcoded values instead of sign protocol constants
+                // used below for readability
+                quint32 newCksum = pktLen & 1 ?
+                    quint16(~origCksum) + quint16(~0x221d) + 0x231d
+                                        + quint16(~0x6100) + (0x6100 | ttagId_) :
+                    quint16(~origCksum) + quint16(~0x0022) + (ttagId_ << 8 | 0x23);
+                while (newCksum > 0xffff)
+                    newCksum = (newCksum & 0xffff) + (newCksum >> 16);
+                // XXX: For IPv4/UDP, if ~newcksum is 0x0000 we are supposed to
+                // set the checksum as 0xffff since 0x0000 indicates no cksum
+                // is present - we choose not to do this to avoid extra cost
+                *cksum = qToBigEndian(quint16(~newCksum));
+            }
+#endif
+            ttagId_++;
+            nextTtagPkt_ += ttagDeltaMarkers_.at(ttagMarkerIndex_);
+            ttagMarkerIndex_++;
+            if (ttagMarkerIndex_ >= ttagDeltaMarkers_.size())
+                ttagMarkerIndex_ = 0;
+        }
+
+        if (sync) {
             long usec = (hdr->ts.tv_sec - ts.tv_sec) * 1000000 +
                 (hdr->ts.tv_usec - ts.tv_usec);
 
             getTimeStamp(&ovrEnd);
-
             overHead -= udiffTimeStamp(&ovrStart, &ovrEnd);
             Q_ASSERT(overHead <= 0);
             usec += overHead;
-            if (usec > 0)
-            {
+            if (usec > 0) {
                 (*udelayFn_)(usec);
                 overHead = 0;
-            }
-            else
+            } else
                 overHead = usec;
 
             ts = hdr->ts;
@@ -453,21 +513,34 @@ int PcapTxThread::sendQueueTransmit(pcap_t *p,
         stats_->pkts++;
         stats_->bytes += pktLen;
 
+        // Revert T-Tag packet changes
+        if (ttagPkt) {
+            *(pkt+pktLen-5) = SignProtocol::kTypeLenTtagPlaceholder;
+            *(pkt+pktLen-6) = 0;
+#if 0
+            if (seq->ttagL4CksumOffset_) {
+                quint16 *cksum = reinterpret_cast<quint16*>(
+                                         pkt + seq->ttagL4CksumOffset_);
+                *cksum = qToBigEndian(origCksum);
+            }
+#endif
+        }
+
         // Step to the next packet in the buffer
         hdr = (struct pcap_pkthdr*) (pkt + pktLen);
         pkt = (uchar*) ((uchar*)hdr + sizeof(*hdr)); // FIXME: superfluous?
 
-        if (stop_)
-        {
+        if (stop_) {
             return -2;
         }
     }
-
     return 0;
 }
 
-void PcapTxThread::updateStreamStats()
+void PcapTxThread::updateTxStreamStats()
 {
+    QMutexLocker lock(&streamStatsLock_);
+
     // If no packets in list, nothing to be done
     if (!packetListSize_)
         return;
@@ -613,4 +686,3 @@ void PcapTxThread::udelay(unsigned long usec)
     QThread::usleep(usec);
 #endif
 }
-
