@@ -24,6 +24,7 @@ along with this program.  If not, see <http://www.gnu.org/licenses/>
 #include "packetsequence.h"
 #include "statstuple.h"
 
+#include <QMutex>
 #include <QThread>
 #include <pcap.h>
 
@@ -42,13 +43,13 @@ public:
     bool appendToPacketList(long sec, long usec, const uchar *packet,
                             int length);
     void setPacketListLoopMode(bool loop, quint64 secDelay, quint64 nsecDelay);
+    bool setPacketListTtagMarkers(QList<uint> markers, uint repeatInterval);
 
     void setHandle(pcap_t *handle);
 
     void setStats(StatsTuple *stats);
 
-    const StreamStats& streamStats();
-    void clearStreamStats();
+    StreamStats streamStats(); // reset on read
 
     void run();
 
@@ -66,9 +67,9 @@ private:
     };
 
     static void udelay(unsigned long usec);
-    int sendQueueTransmit(pcap_t *p, pcap_send_queue *queue, long &overHead,
-                int sync);
-    void updateStreamStats();
+    int sendQueueTransmit(pcap_t *p, PacketSequence *seq,
+            long &overHead, int sync);
+    void updateTxStreamStats();
 
     // Intermediate state variables used while building the packet list
     PacketSequence *currentPacketSequence_;
@@ -80,7 +81,7 @@ private:
     quint64 packetListSize_; // count of pkts in packet List including repeats
 
     int returnToQIdx_;
-    quint64 loopDelay_;
+    quint64 loopDelay_; // in nanosecs
 
     void (*udelayFn_)(unsigned long);
 
@@ -93,8 +94,18 @@ private:
     StatsTuple *stats_;
     StatsTuple lastStats_;
     StreamStats streamStats_;
+    QMutex streamStatsLock_;
+    quint8 ttagId_{0};
 
     double lastTxDuration_{0.0}; // in secs
+
+    // XXX: Ttag Marker config derived; not updated during Tx
+    int firstTtagPkt_;
+    QList<uint> ttagDeltaMarkers_;
+
+    // XXX: Ttag related; updated during Tx
+    int ttagMarkerIndex_;
+    quint64 nextTtagPkt_{0};
 };
 
 #endif
